@@ -12,8 +12,7 @@
 #' @name ogr_manage
 #' @details
 #' These functions are complementary to `ogrinfo()` and `ogr2ogr()` for
-#' vector data management. They are also intended to support vector I/O in a
-#' future release of gdalraster. Bindings to OGR wrap portions of the GDAL
+#' vector data management. Bindings to OGR wrap portions of the GDAL
 #' Vector API (ogr_core.h and ogr_api.h,
 #' \url{https://gdal.org/en/stable/api/vector_c_api.html}).
 #'
@@ -51,8 +50,11 @@
 #' The attribute fields and geometry field(s) to create can be specified as a
 #' feature class definition (`layer_defn` as list, see [ogr_define]), or
 #' alternatively, by giving the `geom_type` and `srs`, optionally along with
-#' one `fld_name` and `fld_type` to be created in the layer. Returns a logical
-#' scalar, `TRUE` indicating success.
+#' one `fld_name` and `fld_type` to be created in the layer.
+#' By default, returns logical `TRUE` indicating success (output written to
+#' `dst_filename`), or an object of class [`GDALVector`][GDALVector] for the
+#' output layer will be returned if `return_obj = TRUE`. An error is raised if
+#' the operation fails.
 #'
 #' `ogr_ds_layer_count()` returns the number of layers in a vector dataset.
 #'
@@ -70,8 +72,8 @@
 #' `RandomRead`, `SequentialWrite`, `RandomWrite`, `UpsertFeature`,
 #' `FastSpatialFilter`, `FastFeatureCount`, `FastGetExtent`,
 #' `FastSetNextByIndex`, `CreateField`, `CreateGeomField`, `DeleteField`,
-#' `ReorderFields`, `AlterFieldDefn`, `AlterGeomFieldDefn`, `DeleteFeature`,
-#' `StringsAsUTF8`, `Transactions`, `CurveGeometries`.
+#' `ReorderFields`, `AlterFieldDefn`, `AlterGeomFieldDefn`, `IgnoreFields`,
+#' `DeleteFeature`, `Rename`, `StringsAsUTF8`, `CurveGeometries`.
 #' See the GDAL documentation for
 #' [`OGR_L_TestCapability()`](https://gdal.org/en/stable/api/vector_c_api.html#_CPPv420OGR_L_TestCapability9OGRLayerHPKc).
 #'
@@ -81,10 +83,19 @@
 #' field names and their definitions (see [ogr_define]).
 #' (Note: use `ogr_ds_create()` to create single-layer formats such as "ESRI
 #' Shapefile", "FlatGeobuf", "GeoJSON", etc.)
-#' Returns a logical scalar, `TRUE` indicating success.
+#' By default, returns logical `TRUE` indicating success, or an object of class
+#' [`GDALVector`][GDALVector] will be returned if `return_obj = TRUE`.
+#' An error is raised if the operation fails.
 #'
 #' `ogr_layer_field_names()` returns a character vector of field names on a
-#' layer, or `NULL` if no fields are found.
+#' layer, or `NULL` if no fields are found. The first layer by index is opened
+#' if `NULL` is given for the `layer` argument.
+#'
+#' `ogr_layer_rename()` renames a layer in a vector dataset. This operation is
+#' implemented only by layers that expose the `Rename` capability (see
+#' `ogr_layer_test_cap()` above). This operation will fail if a layer with the
+#' new name already exists. Returns a logical scalar, `TRUE` indicating success.
+#' Requires GDAL >= 3.5.
 #'
 #' `ogr_layer_delete()` deletes an existing layer in a vector dataset.
 #' Returns a logical scalar, `TRUE` indicating success.
@@ -114,9 +125,10 @@
 #' `ogr_execute_sql()` executes an SQL statement against the data store.
 #' This function can be used to modify the schema or edit data using SQL
 #' (e.g., `ALTER TABLE`, `DROP TABLE`, `CREATE INDEX`, `DROP INDEX`, `INSERT`,
-#' `UPDATE`, `DELETE`). Currently, this function does not return a result set
-#' for a `SELECT` statement. Returns `NULL` invisibly.
-#' Wrapper of `GDALDatasetExecuteSQL()` in the GDAL C API.
+#' `UPDATE`, `DELETE`), or to execute a query (i.e., `SELECT`).
+#' Returns `NULL` (invisibly) for statements that are in error, or that have no
+#' results set, or an object of class `GDALVector` representing a results set
+#' from the query. Wrapper of `GDALDatasetExecuteSQL()` in the GDAL API.
 #'
 #' @param dsn Character string. The vector data source name, e.g., a filename
 #' or database connection string.
@@ -126,6 +138,9 @@
 #' Examples of some common output formats include: `"GPKG"`, `"FlatGeobuf"`,
 #' `"ESRI Shapefile"`, `"SQLite"`.
 #' @param layer Character string for a layer name in a vector dataset.
+#' The `layer` argument may be given as empty string (`""`) in which case the
+#' first layer by index will be opened (except with `ogr_layer_delete()` and
+#' `ogr_layer_rename()` for which a layer name must be specified).
 #' @param layer_defn A feature class definition for `layer` as a list of
 #' zero or more attribute field definitions, and at least one geometry field
 #' definition (see [ogr_define]).
@@ -158,8 +173,6 @@
 #' after the decimal point.
 #' @param is_nullable Optional NOT NULL field constraint (logical scalar).
 #' Defaults to `TRUE`.
-#' @param is_ignored Whether field is ignored when retrieving features (logical
-#' scalar). Defaults to `FALSE`.
 #' @param is_unique Optional UNIQUE constraint on the field (logical scalar).
 #' Defaults to `FALSE`.
 #' @param default_value Optional default value for the field as a character
@@ -182,6 +195,10 @@
 #' given. The `"SQLite"` dialect can also be used (see Note).
 #' @param overwrite Logical scalar. `TRUE` to overwrite `dsn` if it already
 #' exists when calling `ogr_ds_create()`. Default is `FALSE`.
+#' @param return_obj Logical scalar. If `TRUE`, an object of class
+#' [`GDALVector`][GDALVector] open on the newly created layer will be
+#' returned. Defaults to `FALSE`. Must be used with either the `layer` or
+#' `layer_defn` arguments.
 #'
 #' @note
 #' The OGR SQL document linked under **See Also** contains information on the
@@ -206,105 +223,94 @@
 #' dialects for details.
 #'
 #' @seealso
-#' [gdal_formats()], [has_spatialite()], [ogr_def_field()], [ogr_def_layer()],
-#' [ogrinfo()], [ogr2ogr()]
+#  [ogr_define], [ogrinfo()], [ogr2ogr()]
 #'
 #' OGR SQL dialect and SQLite SQL dialect:\cr
 #' \url{https://gdal.org/en/stable/user/ogr_sql_sqlite_dialect.html}
 #'
 #' @examples
+#' ## Create GeoPackage and manage schema
 #' dsn <- file.path(tempdir(), "test1.gpkg")
 #' ogr_ds_create("GPKG", dsn)
 #' ogr_ds_exists(dsn, with_update = TRUE)
-#' ogr_ds_layer_count(dsn)
+#' # dataset capabilities
 #' ogr_ds_test_cap(dsn)
-#' ogr_layer_exists(dsn, "layer1")
-#' if (ogr_ds_test_cap(dsn)$CreateLayer) {
-#'   opt <- c("GEOMETRY_NULLABLE=NO", "DESCRIPTION=test layer")
-#'   ogr_layer_create(dsn, "layer1", geom_type = "Polygon", srs = "EPSG:5070",
-#'                    lco = opt)
-#' }
-#' ogr_ds_layer_count(dsn)
-#' ogr_layer_exists(dsn, "layer1")
-#' ogr_ds_layer_names(dsn)
 #'
-#' ogr_layer_field_names(dsn, "layer1")
-#' ogr_field_index(dsn, "layer1", "field1")
-#' if (ogr_layer_test_cap(dsn, "layer1")$CreateField) {
-#'   ogr_field_create(dsn, "layer1", "field1",
-#'                    fld_type = "OFTInteger64",
-#'                    is_nullable = FALSE)
-#'   ogr_field_create(dsn, "layer1", "field2",
-#'                    fld_type = "OFTString")
-#' }
-#' ogr_field_index(dsn, "layer1", "field1")
-#' ogr_layer_field_names(dsn, "layer1")
+#' ogr_layer_create(dsn, layer = "layer1", geom_type = "Polygon",
+#'                  srs = "EPSG:5070")
+#'
+#' ogr_field_create(dsn, layer = "layer1",
+#'                  fld_name = "field1",
+#'                  fld_type = "OFTInteger64",
+#'                  is_nullable = FALSE)
+#' ogr_field_create(dsn, layer = "layer1",
+#'                  fld_name = "field2",
+#'                  fld_type = "OFTString")
+#'
+#' ogr_ds_layer_count(dsn)
+#' ogr_ds_layer_names(dsn)
+#' ogr_layer_field_names(dsn, layer = "layer1")
 #'
 #' # delete a field
 #' if (ogr_layer_test_cap(dsn, "layer1")$DeleteField) {
-#'   ogr_field_delete(dsn, "layer1", "field2")
+#'   ogr_field_delete(dsn, layer = "layer1", fld_name = "field2")
 #' }
+#'
 #' ogr_layer_field_names(dsn, "layer1")
 #'
-#' # define a feature class (layer definition)
+#' # define a feature class and create layer
 #' defn <- ogr_def_layer("Point", srs = epsg_to_wkt(4326))
 #' # add the attribute fields
-#' defn$fld1_name <- ogr_def_field("OFTInteger64",
-#'                                 is_nullable = FALSE,
-#'                                 is_unique = TRUE)
-#' defn$fld2_name <- ogr_def_field("OFTString",
+#' defn$id_field <- ogr_def_field(fld_type = "OFTInteger64",
+#'                                is_nullable = FALSE,
+#'                                is_unique = TRUE)
+#' defn$str_field <- ogr_def_field(fld_type = "OFTString",
 #'                                 fld_width = 25,
 #'                                 is_nullable = FALSE,
 #'                                 default_value = "'a default string'")
-#' defn$third_field <- ogr_def_field("OFTReal",
-#'                                   default_value = "0.0")
+#' defn$numeric_field <- ogr_def_field(fld_type = "OFTReal",
+#'                                     default_value = "0.0")
 #'
-#' ogr_layer_create(dsn, "layer2", layer_defn = defn)
+#' ogr_layer_create(dsn, layer = "layer2", layer_defn = defn)
 #' ogr_ds_layer_names(dsn)
-#' ogr_layer_field_names(dsn, "layer2")
+#' ogr_layer_field_names(dsn, layer = "layer2")
 #'
 #' # add a field using SQL instead
-#' sql <- "ALTER TABLE layer2 ADD field4 float"
-#' ogr_execute_sql(dsn, sql)
-#' ogr_layer_field_names(dsn, "layer2")
+#' ogr_execute_sql(dsn, sql = "ALTER TABLE layer2 ADD field4 float")
 #'
 #' # rename a field
 #' if (ogr_layer_test_cap(dsn, "layer1")$AlterFieldDefn) {
-#'   ogr_field_rename(dsn, "layer2", "field4", "renamed_field")
+#'   ogr_field_rename(dsn, layer = "layer2",
+#'                    fld_name = "field4",
+#'                    new_name = "renamed_field")
 #' }
-#' ogr_layer_field_names(dsn, "layer2")
+#' ogr_layer_field_names(dsn, layer = "layer2")
 #'
 #' # GDAL >= 3.7
-#' if (as.integer(gdal_version()[2]) >= 3070000)
+#' if (gdal_version_num() >= gdal_compute_version(3, 7, 0))
 #'   ogrinfo(dsn, "layer2")
 #'
-#' deleteDataset(dsn)
-#'
-#' # edit data using SQL
+#' \dontshow{deleteDataset(dsn)}
+#' ## Edit data using SQL
 #' src <- system.file("extdata/ynp_fires_1984_2022.gpkg", package="gdalraster")
 #' perims_shp <- file.path(tempdir(), "mtbs_perims.shp")
-#' ogr2ogr(src, perims_shp, src_layers = "mtbs_perims")
-#' ogr_ds_format(perims_shp)
-#' ogr_ds_layer_names(perims_shp)
-#' ogr_layer_field_names(perims_shp, "mtbs_perims")
+#' ogr2ogr(src_dsn = src, dst_dsn = perims_shp, src_layers = "mtbs_perims")
+#' ogr_ds_format(dsn = perims_shp)
+#' ogr_ds_layer_names(dsn = perims_shp)
+#' ogr_layer_field_names(dsn = perims_shp, layer = "mtbs_perims")
 #'
-#' if (ogr_layer_test_cap(perims_shp, "mtbs_perims")$CreateField) {
-#'   sql <- "ALTER TABLE mtbs_perims ADD burn_bnd_ha float"
-#'   ogr_execute_sql(perims_shp, sql)
-#'   # with GDAL >= 3.7, equivalent to:
-#'   # ogrinfo(perims_shp, cl_arg = c("-sql", sql), read_only = FALSE)
-#' }
+#' alt_tbl <- "ALTER TABLE mtbs_perims ADD burn_bnd_ha float"
+#' ogr_execute_sql(dsn = perims_shp, sql = alt_tbl)
 #'
-#' sql <- "UPDATE mtbs_perims SET burn_bnd_ha = (burn_bnd_ac / 2.471)"
-#' ogr_execute_sql(perims_shp, sql, dialect = "SQLite")
-#' ogr_layer_field_names(perims_shp, "mtbs_perims")
+#' upd <- "UPDATE mtbs_perims SET burn_bnd_ha = (burn_bnd_ac / 2.471)"
+#' ogr_execute_sql(dsn = perims_shp, sql = upd, dialect = "SQLite")
+#' ogr_layer_field_names(dsn = perims_shp, layer = "mtbs_perims")
 #'
 #' # if GDAL >= 3.7:
-#' #   ogrinfo(perims_shp, "mtbs_perims")
+#' #   ogrinfo(dsn = perims_shp, layer = "mtbs_perims")
 #' # or, for output incl. the feature data (omit the default "-so" arg):
-#' #   ogrinfo(perims_shp, "mtbs_perims", cl_arg = "-nomd")
-#'
-#' deleteDataset(perims_shp)
+#' #   ogrinfo(dsn = perims_shp, layer = "mtbs_perims", cl_arg = "-nomd")
+#' \dontshow{deleteDataset(perims_shp)}
 #' @export
 ogr_ds_exists <- function(dsn, with_update = FALSE) {
     if (!(is.character(dsn) && length(dsn) == 1))
@@ -340,38 +346,89 @@ ogr_ds_test_cap <- function(dsn, with_update = TRUE) {
 ogr_ds_create <- function(format, dsn, layer = NULL, layer_defn = NULL,
                           geom_type = NULL, srs = NULL, fld_name = NULL,
                           fld_type = NULL, dsco = NULL, lco = NULL,
-                          overwrite = FALSE) {
+                          overwrite = FALSE, return_obj = FALSE) {
 
+    # format
     if (!(is.character(format) && length(format) == 1))
         stop("'format' must be a length-1 character vector", call. = FALSE)
+    # dsn
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
     if (vsi_stat(dsn) && !overwrite)
-        stop("'dsn' exists and 'overwrite' is FALSE", call. = FALSE)
+        stop("'dsn' exists but 'overwrite' is `FALSE`", call. = FALSE)
+    # layer
     if (is.null(layer))
         layer <- ""
     if (!(is.character(layer) && length(layer) == 1))
         stop("'layer' must be a length-1 character vector", call. = FALSE)
+    # layer_defn
+    if (!is.null(layer_defn)) {
+        if (!is.list(layer_defn) || is.data.frame(layer_defn)) {
+            stop("'layer_defn' must be a list object", call. = FALSE)
+        }
+    }
+    # geom_type
     if (is.null(geom_type))
         geom_type <- ""
     if (!(is.character(geom_type) && length(geom_type) == 1))
         stop("'geom_type' must be a length-1 character vector", call. = FALSE)
+    # srs
     if (is.null(srs))
         srs <- ""
+    if (!(is.character(srs) && length(srs) == 1))
+        stop("'srs' must be a length-1 character vector", call. = FALSE)
+    # fld_name
     if (is.null(fld_name))
         fld_name <- ""
+    if (!(is.character(fld_name) && length(fld_name) == 1))
+        stop("'fld_name' must be a length-1 character vector", call. = FALSE)
+    # fld_type
     if (is.null(fld_type))
         fld_type <- ""
+    if (!(is.character(fld_type) && length(fld_type) == 1))
+        stop("'fld_type' must be a length-1 character vector", call. = FALSE)
+    # dsco
+    if (!is.null(dsco)) {
+        if (!is.character(dsco))
+            stop("'dsco' must be a character vector", call. = FALSE)
+    }
+    # lco
+    if (!is.null(lco)) {
+        if (!is.character(lco))
+            stop("'lco' must be a character vector", call. = FALSE)
+    }
+    # overwrite
+    if (is.null(overwrite))
+        stop("'overwrite' must be a logical value", call. = FALSE)
+    if (!is.logical(overwrite) || length(overwrite) > 1)
+        stop("'overwrite' must be a logical scalar", call. = FALSE)
+    # return_obj
+    if (is.null(return_obj))
+        stop("'return_obj' must be a logical value", call. = FALSE)
+    if (!is.logical(return_obj) || length(return_obj) > 1)
+        stop("'return_obj' must be a logical scalar", call. = FALSE)
+    if (return_obj && layer == "" && is.null(layer_defn)) {
+        stop("'layer' or 'layer_defn' must be given with 'return_obj = TRUE'",
+             call. = FALSE)
+    }
 
     if (is.null(layer_defn)) {
-        return(.create_ogr(format, dsn, 0, 0, 0, "Unknown",
-                           layer, geom_type, srs, fld_name, fld_type,
-                           dsco, lco, NULL))
+        # signature for create_ogr() object factory
+        lyr <- new(GDALVector, format, dsn, layer, geom_type, srs, fld_name,
+                   fld_type, dsco, lco, NULL)
+
     } else {
-        return(.create_ogr(format, dsn, 0, 0, 0, "Unknown",
-                           layer = layer, geom_type = "", srs = "",
-                           fld_name = "", fld_type = "",
-                           dsco = dsco, lco = lco, layer_defn = layer_defn))
+        # signature for create_ogr() object factory
+        lyr <- new(GDALVector, format, dsn, layer = layer, geom_type = "",
+                   srs = "", fld_name = "", fld_type = "", dsco = dsco,
+                   lco = lco, layer_defn = layer_defn)
+    }
+
+    if (return_obj) {
+        return(lyr)
+    } else {
+        lyr$close()
+        return(TRUE)
     }
 }
 
@@ -406,9 +463,11 @@ ogr_layer_exists <- function(dsn, layer) {
 
 #' @name ogr_manage
 #' @export
-ogr_layer_test_cap <- function(dsn, layer, with_update = TRUE) {
+ogr_layer_test_cap <- function(dsn, layer = NULL, with_update = TRUE) {
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
+    if (is.null(layer))
+        layer <- ""
     if (!(is.character(layer) && length(layer) == 1))
         stop("'layer' must be a length-1 character vector", call. = FALSE)
 
@@ -418,8 +477,9 @@ ogr_layer_test_cap <- function(dsn, layer, with_update = TRUE) {
 #' @name ogr_manage
 #' @export
 ogr_layer_create <- function(dsn, layer, layer_defn = NULL, geom_type = NULL,
-                             srs = NULL, lco = NULL) {
+                             srs = NULL, lco = NULL, return_obj = FALSE) {
 
+    # dsn / layer
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
     if (!(is.character(layer) && length(layer) == 1))
@@ -428,11 +488,14 @@ ogr_layer_create <- function(dsn, layer, layer_defn = NULL, geom_type = NULL,
         stop("'dsn' does not exist, or no update access", call. = FALSE)
     if (ogr_layer_exists(dsn, layer))
         stop("'layer' already exists", call. = FALSE)
+    # layer_defn
+    if (!is.null(layer_defn)) {
+        if (!is.list(layer_defn) || is.data.frame(layer_defn)) {
+            stop("'layer_defn' must be a list object", call. = FALSE)
+        }
+    }
 
     if (!is.null(layer_defn)) {
-        if (!is.list(layer_defn))
-            stop("'layer_defn' must be a list", call. = FALSE)
-
         # using layer_defn so get geom_type and srs from the geom field defn
         has_geom_fld_defn <- FALSE
         for (nm in names(layer_defn)) {
@@ -458,31 +521,63 @@ ogr_layer_create <- function(dsn, layer, layer_defn = NULL, geom_type = NULL,
         }
     }
 
+    # geom_type
     if (is.null(geom_type)) {
-        message("geometry type not specified, using 'UNKNOWN'", call. = FALSE)
+        warning("geometry type not specified, using 'UNKNOWN'", call. = FALSE)
         geom_type <- "UNKNOWN"
     }
-
+    # srs
     if (is.null(srs))
         srs <- ""
+    if (!(is.character(srs) && length(srs) == 1))
+        stop("'srs' must be a length-1 character vector", call. = FALSE)
+    # lco
+    if (!is.null(lco)) {
+        if (!is.character(lco))
+            stop("'lco' must be a character vector", call. = FALSE)
+    }
+    # return_obj
+    if (is.null(return_obj))
+        stop("'return_obj' must be a logical value", call. = FALSE)
+    if (!is.logical(return_obj) || length(return_obj) > 1)
+        stop("'return_obj' must be a logical scalar", call. = FALSE)
 
-    return(.ogr_layer_create(dsn = dsn,
-                             layer = layer,
-                             layer_defn = layer_defn,
-                             geom_type = geom_type,
-                             srs = srs,
-                             options = lco))
+    # signature for ogr_layer_create() object factory
+    lyr <- new(GDALVector, dsn, layer, layer_defn, geom_type, srs, lco, TRUE)
+
+    if (return_obj) {
+        return(lyr)
+    } else {
+        lyr$close()
+        return(TRUE)
+    }
 }
 
 #' @name ogr_manage
 #' @export
-ogr_layer_field_names <- function(dsn, layer) {
+ogr_layer_field_names <- function(dsn, layer = NULL) {
+    if (!(is.character(dsn) && length(dsn) == 1))
+        stop("'dsn' must be a length-1 character vector", call. = FALSE)
+    if (is.null(layer))
+        layer <- ""
+    if (!(is.character(layer) && length(layer) == 1))
+        stop("'layer' must be a length-1 character vector", call. = FALSE)
+
+
+    return(.ogr_layer_field_names(dsn, layer))
+}
+
+#' @name ogr_manage
+#' @export
+ogr_layer_rename <- function(dsn, layer, new_name) {
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
     if (!(is.character(layer) && length(layer) == 1))
         stop("'layer' must be a length-1 character vector", call. = FALSE)
+    if (!(is.character(new_name) && length(new_name) == 1))
+        stop("'new_name' must be a length-1 character vector", call. = FALSE)
 
-    return(.ogr_layer_field_names(dsn, layer))
+    return(.ogr_layer_rename(dsn, layer, new_name))
 }
 
 #' @name ogr_manage
@@ -518,7 +613,6 @@ ogr_field_create <- function(dsn, layer, fld_name,
                              fld_width = 0L,
                              fld_precision = 0L,
                              is_nullable = TRUE,
-                             is_ignored = FALSE,
                              is_unique = FALSE,
                              default_value = "") {
 
@@ -568,11 +662,6 @@ ogr_field_create <- function(dsn, layer, fld_name,
         else
             is_nullable <- TRUE
 
-        if (!is.null(fld_defn$is_ignored))
-            is_ignored <- fld_defn$is_ignored
-        else
-            is_ignored <- FALSE
-
         if (!is.null(fld_defn$is_unique))
             is_unique <- fld_defn$is_unique
         else
@@ -585,8 +674,8 @@ ogr_field_create <- function(dsn, layer, fld_name,
     }
 
     return(.ogr_field_create(dsn, layer, fld_name, fld_type, fld_subtype,
-                             fld_width, fld_precision, is_nullable,
-                             is_ignored, is_unique, default_value))
+                             fld_width, fld_precision, is_nullable, is_unique,
+                             default_value))
 }
 
 #' @name ogr_manage
@@ -595,8 +684,7 @@ ogr_geom_field_create <- function(dsn, layer, fld_name,
                                   geom_fld_defn = NULL,
                                   geom_type = NULL,
                                   srs = NULL,
-                                  is_nullable = TRUE,
-                                  is_ignored = FALSE) {
+                                  is_nullable = TRUE) {
 
     if (!(is.character(dsn) && length(dsn) == 1))
         stop("'dsn' must be a length-1 character vector", call. = FALSE)
@@ -643,15 +731,10 @@ ogr_geom_field_create <- function(dsn, layer, fld_name,
             is_nullable <- geom_fld_defn$is_nullable
         else
             is_nullable <- TRUE
-
-        if (!is.null(geom_fld_defn$is_ignored))
-            is_ignored <- geom_fld_defn$is_ignored
-        else
-            is_ignored <- FALSE
     }
 
     return(.ogr_geom_field_create(dsn, layer, fld_name, geom_type, srs,
-                                  is_nullable, is_ignored))
+                                  is_nullable))
 }
 
 #' @name ogr_manage
@@ -707,5 +790,11 @@ ogr_execute_sql <- function(dsn, sql, spatial_filter = NULL, dialect = NULL) {
         dialect <- ""
     }
 
-    return(.ogr_execute_sql(dsn, sql, spatial_filter, dialect))
+    if (startsWith(toupper(sql), "SELECT ")) {
+        return(new(GDALVector, dsn, sql, read_only = TRUE,
+                   open_options = NULL, spatial_filter = spatial_filter,
+                   dialect = dialect))
+    } else {
+        return(.ogr_execute_sql(dsn, sql, spatial_filter, dialect))
+    }
 }
