@@ -19,11 +19,17 @@ void setPROJEnableNetwork(int enabled);
 
 Rcpp::NumericMatrix inv_project(const Rcpp::RObject &pts,
                                 const std::string &srs,
-                                std::string well_known_gcs);
+                                const std::string &well_known_gcs);
 
 Rcpp::NumericMatrix transform_xy(const Rcpp::RObject &pts,
                                  const std::string &srs_from,
                                  const std::string &srs_to);
 
+Rcpp::NumericVector transform_bounds(const Rcpp::NumericVector &bbox,
+                                     const std::string &srs_from,
+                                     const std::string &srs_to,
+                                     int densify_pts,
+                                     bool traditional_gis_order);
 
 #endif  // SRC_TRANSFORM_H_
+
